@@ -1,10 +1,13 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Sun Nov 11 11:20:42 2018
-
-@author: yoelr
 """
-from thermosteam.base.units_of_measure import DisplayUnits, convert
+from thermosteam.units_of_measure import DisplayUnits, convert
 
 __all__ = ('PowerUtility',)
 
@@ -57,7 +60,8 @@ class PowerUtility:
         return bool(self.rate)
     
     def __call__(self, rate):
-        """Calculate cost and save results. 
+        """
+        Calculate cost and save results. 
         
         Parameters
         ----------
@@ -68,6 +72,22 @@ class PowerUtility:
         self.rate = rate
         self.cost = self.price * rate
     
+    def mix_from(self, power_utilities):
+        """
+        Mix in requirements of power utilities.
+        """
+        self.rate = sum([i.rate for i in power_utilities])
+        self.cost = sum([i.cost for i in power_utilities])
+    
+    @classmethod
+    def sum(cls, power_utilities):
+        """
+        Return a PowerUtility object that represents the sum of power utilities.
+        """
+        power_utility = cls()
+        power_utility.mix_from(power_utilities)
+        return power_utility
+    
     def show(self, rate=None, cost=None):
         # Get units of measure
         display_units = self.display_units
