@@ -1,15 +1,17 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Mon Feb  4 19:38:37 2019
-
-@author: Guest Group
 """
-
 import pandas as pd
 import numpy as np
-from flexsolve import wegstein_secant, aitken_secant, secant
+import flexsolve as flx
 from copy import copy as copy_
-from numba import njit
+from flexsolve import njitable
 
 __all__ = ('TEA', 'CombinedTEA')
 
@@ -47,17 +49,34 @@ _MACRS = {'MACRS5': np.array([.2000, .3200, .1920,
 
 
 # %% Utilities
-    
-@njit    
+
+@njitable(cache=True)
+def NPV_at_IRR(IRR, cashflow, duration_array):
+    """Return NPV at given IRR and cashflow data."""
+    return (cashflow/(1.+IRR)**duration_array).sum()
+
+@njitable(cache=True)
+def NPV_with_sales(sales, NPV, coefficients, discount_factors):
+    """Return NPV with an additional annualized sales."""
+    return NPV + (sales*coefficients/discount_factors).sum()
+
+@njitable(cache=True)
+def sum_NPV_at_IRR(IRR, cashflows, duration_arrays):
+    NPV = 0.
+    for i in range(len(cashflows)):
+        NPV += NPV_at_IRR(IRR, cashflows[i], duration_arrays[i])
+    return NPV
+
+@njitable(cache=True)
 def initial_loan_principal(loan, interest):
     principal = 0
     k = 1. + interest
-    for loan_i in loan:
-        principal += loan_i 
+    for i in loan:
+        principal += i
         principal *= k
     return principal
 
-@njit
+@njitable(cache=True)
 def final_loan_principal(payment, principal, interest, years):
     for iter in range(years):
         principal += principal * interest - payment
@@ -65,11 +84,14 @@ def final_loan_principal(payment, principal, interest, years):
 
 def solve_payment(payment, loan, interest, years):
     principal = initial_loan_principal(loan, interest)
-    return wegstein_secant(final_loan_principal,
-                           payment, payment+10., 1e-4, 1e-4,
-                           args=(principal, interest, years))
+    payment = flx.aitken_secant(final_loan_principal,
+                                payment, payment+10., 1e-6, 1.,
+                                args=(principal, interest, years),
+                                maxiter=200,
+                                checkroot=False)
+    return payment
 
-@njit
+@njitable(cache=True)
 def net_earnings(D, C, S, start,
                  FCI, TDC, VOC, FOC, sales,
                  income_tax,
@@ -149,7 +171,7 @@ class TEA:
     
     Examples
     --------
-    :doc:`tutorial/Techno-economic analysis` 
+    :doc:`tutorial/Techno-economic_analysis` 
 
     """
     
@@ -170,11 +192,10 @@ class TEA:
 
     @staticmethod
     def like(system, other):
-        """Create a Cashflow object from `system` with the same settings as `other`."""
+        """Create a TEA object from `system` with the same settings as `other`."""
         self = copy_(other)
         self.units = sorted(system._costunits, key=lambda x: x.line)
         self.system = system
-        system._TEA = self
         return self
 
     def __init__(self, system, IRR, duration, depreciation, income_tax,
@@ -288,13 +309,13 @@ class TEA:
         """Total purchase cost (USD)."""
         return sum([u.purchase_cost for u in self.units])
     @property
-    def installation_cost(self):
-        """Total installation cost (USD)."""
-        return sum([u.installation_cost for u in self.units])
+    def installed_cost(self):
+        """Total installed cost (USD)."""
+        return sum([u.installed_cost for u in self.units])
     @property
     def DPI(self):
         """Direct permanent investment."""
-        return self.purchase_cost * self.lang_factor if self.lang_factor else self.installation_cost
+        return self.purchase_cost * self.lang_factor if self.lang_factor else self.installed_cost
     @property
     def TDC(self):
         """Total depreciable capital."""
@@ -400,11 +421,11 @@ class TEA:
         if self.finance_interest:
             interest = self.finance_interest
             years = self.finance_years
-            end = start+years
+            end = start + years
             L[:start] = loan = self.finance_fraction*(C_FC[:start]+C_WC[:start])
-            f_interest = (1.+interest)
+            f_interest = (1. + interest)
             LP[start:end] = solve_payment(loan.sum()/years * f_interest,
-                                           loan, interest, years)
+                                          loan, interest, years)
             loan_principal = 0
             for i in range(end):
                 LI[i] = li = (loan_principal + L[i]) * interest 
@@ -435,7 +456,7 @@ class TEA:
     @property
     def NPV(self):
         """Net present value."""
-        return self._NPV_at_IRR(self.IRR, self.cashflow)
+        return NPV_at_IRR(self.IRR, self.cashflow, self._duration_array)
     
     def _AOC(self, FCI):
         """Return AOC at given FCI"""
@@ -446,8 +467,8 @@ class TEA:
         
         Parameters
         ----------
-        products : Iterable[Stream]
-                    Main products of the system
+        products : Iterable[:class:`~thermosteam.Stream`]
+            Main products of the system
         with_annual_depreciation=True : bool, optional
         
         Notes
@@ -467,7 +488,7 @@ class TEA:
         
         Parameters
         ----------
-        products : Iterable[Stream]
+        products : Iterable[:class:`~thermosteam.Stream`]
                     Main products of the system
         with_annual_depreciation=True : bool, optional
         
@@ -523,28 +544,16 @@ class TEA:
             return NE + D + Loan - C_FC - C_WC - LP
         else:
             return NE + D - C_FC - C_WC
-    
-    def _NPV_at_IRR(self, IRR, cashflow):
-        """Return NPV at given IRR and cashflow data."""
-        return (cashflow/(1.+IRR)**self._duration_array).sum()
-
-    def _NPV_with_sales(self, sales, NPV, coefficients, discount_factors):
-        """Return NPV with an additional annualized sales."""
-        return NPV + (sales*coefficients/discount_factors).sum()
 
     def solve_IRR(self):
         """Return the IRR at the break even point (NPV = 0) through cash flow analysis."""
-        try:
-            self._IRR = wegstein_secant(self._NPV_at_IRR,
-                                        self._IRR, self._IRR+1e-6,
-                                        xtol=1e-6, maxiter=200,
-                                        args=(self.cashflow,))
-        except:
-            self._IRR = secant(self._NPV_at_IRR,
-                               0.15, 0.15001,
-                               xtol=1e-6, maxiter=200,
-                               args=(self.cashflow,))
-        return self._IRR
+        IRR = self._IRR
+        args = (self.cashflow, self._duration_array)
+        IRR = flx.aitken_secant(NPV_at_IRR,
+                                IRR, 1.0001 * IRR + 1e-3,
+                                maxiter=200, args=args, checkroot=False)
+        self._IRR = IRR
+        return IRR
     
     def _price2cost(self, stream):
         """Get factor to convert stream price to cost for cashflow in solve_price method."""
@@ -555,33 +564,29 @@ class TEA:
         
         Parameters
         ----------
-        stream : Stream
+        stream : :class:`~thermosteam.Stream`
             Stream with variable selling price.
             
         """
         price2cost = self._price2cost(stream)
-        discount_factors = (1 + self.IRR)**self._duration_array
         cashflow = self.cashflow
+        discount_factors = (1 + self.IRR)**self._duration_array
         NPV = (cashflow/discount_factors).sum()
         coefficients = np.ones_like(discount_factors)
         start = self._start
         coefficients[:start] = 0
         w0 = self._startup_time
         coefficients[self._start] =  w0*self.startup_VOCfrac + (1-w0)
-        try:
-            self._sales = wegstein_secant(self._NPV_with_sales,
-                                          self._sales, self._sales+1e-6,
-                                          xtol=1e-6, maxiter=200,
-                                          args=(NPV, coefficients, discount_factors))
-        except:
-            self._sales = secant(self._NPV_with_sales,
-                                 0, 1e-6,
-                                 xtol=1e-6, maxiter=200,
-                                 args=(NPV, coefficients, discount_factors))
+        args = (NPV, coefficients, discount_factors)
+        sales = self._sales or stream.price * price2cost
+        sales = flx.aitken_secant(NPV_with_sales,
+                                  sales, 1.0001 * sales + 1e-3,
+                                  maxiter=200, args=args, checkroot=False)
+        self._sales = sales
         if stream.sink:
-            return stream.price - self._sales/price2cost
+            return stream.price - sales/price2cost
         elif stream.source:
-            return stream.price + self._sales/price2cost
+            return stream.price + sales/price2cost
         else:
             raise ValueError(f"stream must be either a feed or a product")
     
@@ -601,13 +606,28 @@ class TEA:
                 
     
 class CombinedTEA(TEA):
+    """
+    Create a CombinedTEA object that performs techno-economic analysis by 
+    using data from many TEA objects that correspond to different areas
+    of a biorefinery. A CombinedTEA object serves to accomodate for areas of
+    a biorefinery with different assumptions. For example, an area of a
+    biorefinery may not operate the same days a year, may have different 
+    depreciation schedules, and may even be taxed differently than other 
+    areas. Ultimately, a CombinedTEA object is an aggregation of TEA objects
+    that work together to conduct techno-economic analysis of a whole biorefinery.
     
+    Parameters
+    ----------
+    TEAs : Iterable[TEA]
+        TEA objects used to conduct techno-economic analysis.
+    
+    """
     _TDC = _FCI = _FOC = NotImplemented
     
     __slots__ = ('TEAs',)
     
     def __init__(self, TEAs, IRR):
-        #: iterable[TEA] All TEA objects for cashflow calculation
+        #: Iterable[TEA] All TEA objects for cashflow calculation
         self.TEAs = TEAs
         
         #: [float] Internal rate of return (fraction)
@@ -664,12 +684,14 @@ class CombinedTEA(TEA):
         """Total purchase cost (USD)."""
         return sum([i.purchase_cost for i in self.TEAs])
     @property
-    def installation_cost(self):
+    def installed_cost(self):
         """Total installation cost (USD)."""
-        return sum([i.installation_cost for i in self.TEAs])
+        return sum([i.installed_cost for i in self.TEAs])
     @property
     def NPV(self):
-        return sum([i.NPV for i in self.TEAs])
+        return sum_NPV_at_IRR(self.IRR,
+                              tuple([i.cashflow for i in self.TEAs]),
+                              tuple([i._duration_array for i in self.TEAs]))
     @property
     def DPI(self):
         """Direct permanent investment."""
@@ -726,34 +748,31 @@ class CombinedTEA(TEA):
         """Pay back period (yr) without accounting for annualized depreciation."""
         return self.FCI/self.net_earnings
     
-    def get_cashflow(self):
+    def get_cashflow_table(self):
         """Return DataFrame of the cash flow analysis."""
-        TEA, *TEAs = self.TEAs
-        table = TEA.get_cashflow()
+        IRR = self.IRR
+        TEAs = self.TEAs
+        IRRs = [i.IRR for i in TEAs]
+        for i in TEAs: i.IRR = IRR
+        TEA, *other_TEAs = TEAs
+        table = TEA.get_cashflow_table()
         DF = table['Discount factor']
         DF_data = np.array(DF)
-        for i in TEAs:
-            i_table = i.get_cashflow()
+        for i in other_TEAs:
+            i_table = i.get_cashflow_table()
             if (i_table.index != table.index).any():
-                raise NotImplementedError('cannot create cashflow table from TEAs with different venture years')
+                raise NotImplementedError('cannot yet create cashflow table from TEAs with different venture years')
             table[:] += np.asarray(i_table)
         DF[:] = DF_data
+        for IRR, TEA in zip(IRRs, TEAs): TEA.IRR = IRR
         return table    
-        
-    def _NPV_at_IRR(self, IRR, TEA_cashflows):
-        """Return NPV at given IRR and cashflow data."""
-        return sum([i._NPV_at_IRR(IRR, j) for i, j in TEA_cashflows])
-
-    def _NPV_with_sales(self, sales, NPV, TEA, coefficients, discount_factors):
-        """Return NPV with additional sales."""
-        return TEA._NPV_with_sales(sales, NPV, coefficients, discount_factors)
     
     def production_cost(self, products, with_annual_depreciation=True):
         """Return production cost of products [USD/yr].
         
         Parameters
         ----------
-        products : Iterable[Stream]
+        products : Iterable[:class:`~thermosteam.Stream`]
                     Main products of the system
         with_annual_depreciation=True : bool, optional
         
@@ -774,18 +793,12 @@ class CombinedTEA(TEA):
     
     def solve_IRR(self):
         """Return the IRR at the break even point (NPV = 0) through cash flow analysis."""
-        try:
-            self._IRR = aitken_secant(self._NPV_at_IRR,
-                                      self._IRR, self._IRR+1e-6,
-                                      xtol=1e-8, maxiter=200,
-                                      args=([(i, i.cashflow)
-                                             for i in self.TEAs],))
-        except:
-            self._IRR = secant(self._NPV_at_IRR,
-                               0.15, 0.15001,
-                               xtol=5e-8, maxiter=200,
-                               args=([(i, i.cashflow) 
-                                      for i in self.TEAs],))
+        IRR = self._IRR
+        args = (tuple([i.cashflow for i in self.TEAs]),
+                tuple([i._duration_array for i in self.TEAs]))
+        self._IRR = flx.aitken_secant(sum_NPV_at_IRR,
+                                      IRR, 1.0001 * IRR + 1e-3,
+                                      maxiter=200, args=args, checkroot=False)
         return self._IRR
     
     def solve_price(self, stream, TEA=None):
@@ -793,42 +806,35 @@ class CombinedTEA(TEA):
         
         Parameters
         ----------
-        stream : Stream
+        stream : :class:`~thermosteam.Stream`
                  Stream with variable selling price.
         TEA : TEA, optional
               Stream should belong here.
             
         """
+        TEAs = self.TEAs
         if not TEA:
-            for TEA in self.TEAs:
+            for TEA in TEAs:
                 if stream in TEA.system.feeds or stream in TEA.system.products: break
         price2cost = TEA._price2cost(stream)
         IRR = self.IRR
-        NPV = sum([i._NPV_at_IRR(IRR, i.cashflow) for i in self.TEAs])
+        NPV = self.NPV
         discount_factors = (1.+IRR)**TEA._duration_array
         coefficients = np.ones_like(discount_factors)
         start = TEA._start
         coefficients[:start] = 0
         w0 = TEA._startup_time
         coefficients[TEA._start] =  w0*TEA.startup_VOCfrac + (1-w0)
-        try:
-            self._sales = aitken_secant(self._NPV_with_sales,
-                                        self._sales, self._sales+1e-6,
-                                        xtol=5e-8, maxiter=200,
-                                        args=(NPV, TEA,
-                                              coefficients,
-                                              discount_factors))
-        except:
-            self._sales = secant(self._NPV_with_sales,
-                                 0, 1e-6,
-                                 xtol=5e-8, maxiter=200,
-                                 args=(NPV, TEA,
-                                       coefficients,
-                                       discount_factors))
+        args = (NPV, coefficients, discount_factors)
+        sales = self._sales or stream.price * price2cost
+        sales = flx.aitken_secant(NPV_with_sales,
+                                  sales, 1.0001 * sales + 1e-3,
+                                  maxiter=200, args=args, checkroot=False)
+        self._sales = sales
         if stream.sink:
-            return stream.price - self._sales/price2cost
+            return stream.price - sales/price2cost
         elif stream.source:
-            return stream.price + self._sales/price2cost
+            return stream.price + sales/price2cost
         else:
             raise ValueError(f"stream must be either a feed or a product")
     
