@@ -1,12 +1,17 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:14:01 2018
-
-@author: yoelr
 """
 from ._solids_separator import SolidsSeparator
 from .decorators import cost
 
+__all__ = ('CrushingMill',)
+
 @cost('Flow rate', units='kg/hr', cost=1.5e6, CE=541.7,
       n=0.6, S=335e3, kW=2010, BM=2.3)
 class CrushingMill(SolidsSeparator):
