@@ -1,27 +1,30 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:45:47 2018
-
-@author: yoelr
 """
 import numpy as np
-from ._hx import HXutility
-from .. import Unit
+from ._batch_bioreactor import BatchBioreactor
 from scipy.integrate import odeint
-from .decorators import cost
-from ._tank import MixTank
-from .design_tools import size_batch
 from thermosteam.reaction import Reaction
 
-@cost('Reactor volume', 'Cleaning in place', CE=521.9,
-      cost=421e3, S=3785, n=0.6, BM=1.8, N='N')
-@cost('Reactor volume', 'Agitators', CE=521.9, cost=52500,
-      S=3785, n=0.5, kW=22.371, BM=1.5, N='N')
-@cost('Reactor volume', 'Reactors', CE=521.9, cost=844000,
-      S=3785, n=0.5, BM=1.5, N='N')
-class Fermentation(Unit):
+__all__ = ('Fermentation',)
+
+class Fermentation(BatchBioreactor):
     """
-    Create a Fermentation object which models large-scale batch fermentation for the production of 1st generation ethanol using yeast [1]_ [2]_ [3]_ [4]_. A compound with CAS 'Yeast' must be present in species. Only sucrose and glucose are taken into account for conversion. Conversion is based on reaction time, `tau`. Cleaning and unloading time, `tau_0`, fraction of working volume, `V_wf`, and number of reactors, `N_reactors`, are attributes that can be changed. Cost of a reactor is based on the NREL batch fermentation tank cost assuming volumetric scaling with a 6/10th exponent [5]_. 
+    Create a Fermentation object which models large-scale batch fermentation
+    for the production of 1st generation ethanol using yeast
+    [1]_ [2]_ [3]_ [4]_. A compound with CAS 'Yeast' must be present.
+    Only sucrose and glucose are taken into account for conversion.
+    Conversion is based on reaction time, `tau`. Cleaning and unloading time,
+    `tau_0`, fraction of working volume, `V_wf`, and number of reactors,
+    `N_reactors`, are attributes that can be changed. Cost of a reactor
+    is based on the NREL batch fermentation tank cost assuming volumetric
+    scaling with a 6/10th exponent [5]_. 
     
     Parameters
     ----------
@@ -32,29 +35,46 @@ class Fermentation(Unit):
         * [1] Effluent
     tau : float
         Reaction time.
+    N : int, optional
+        Number of batch reactors
+    V : float, optional
+        Target volume of reactors [m^3].
+    T=305.15 : float
+        Temperature of reactor [K].
+    P=101325 : float
+        Operating pressure of reactor [Pa].
+    Nmin=2 : int
+        Minimum number of fermentors.
+    Nmax=36: int
+        Maximum number of fermentors.  
     efficiency=0.9 : float, optional
         User enforced efficiency.
     iskinetic=False: bool, optional
         If True, `Fermenation.kinetic_model` will be used.
-    N : int
-        Number of batch reactors
+    
+    Notes
+    -----
+    Either N or V must be given.
     
     Examples
     --------
-    Simulate a Fermentation object which models batch fermentation for the production of 1st generation ethanol using yeast.
+    Simulate a Fermentation object which models batch fermentation for the
+    production of 1st generation ethanol using yeast.
     
     >>> from biorefineries.lipidcane.chemicals import ethanol_chemicals 
     >>> from biosteam.units import Fermentation
-    >>> from thermosteam import Stream, settings
+    >>> from biosteam import Stream, settings
     >>> settings.set_thermo(ethanol_chemicals)
     >>> feed = Stream('feed',
-    ...              Water=1.20e+05,
-    ...              Glucose=1.89e+03,
-    ...              Sucrose=2.14e+04,
-    ...              DryYeast=1.03e+04,
-    ...              units='kg/hr',
-    ...              T=32+273.15)
-    >>> F1 = Fermentation('F1', ins=feed, outs=('CO2', 'product'), tau=8, efficiency=0.90, N=8)
+    ...               Water=1.20e+05,
+    ...               Glucose=1.89e+03,
+    ...               Sucrose=2.14e+04,
+    ...               DryYeast=1.03e+04,
+    ...               units='kg/hr',
+    ...               T=32+273.15)
+    >>> F1 = Fermentation('F1',
+    ...                   ins=feed, outs=('CO2', 'product'),
+    ...                   tau=8, efficiency=0.90, N=8)
     >>> F1.simulate()
     >>> F1.show()
     Fermentation: F1
@@ -68,37 +88,36 @@ class Fermentation(Unit):
     [1] missing stream
     outs...
     [0] CO2
-        phase: 'g', T: 305.15 K, P: 101325 Pa
-        flow (kmol/hr): Water    2.5
+        phase: 'g', T: 298.15 K, P: 101325 Pa
+        flow (kmol/hr): Water    1.66
                         CO2      244
-                        Ethanol  0.582
+                        Ethanol  0.382
     [1] product
         phase: 'l', T: 305.15 K, P: 101325 Pa
         flow (kmol/hr): Water     6.6e+03
-                        Ethanol   243
+                        Ethanol   244
                         Glucose   13.6
                         DryYeast  1.03e+04
-    
     >>> F1.results()
     Fermentation                                       Units        F1
-    Power               Rate                              kW      11.5
-                        Cost                          USD/hr     0.899
-    Chilled water       Duty                           kJ/hr -1.35e+07
-                        Flow                         kmol/hr  9.02e+03
-                        Cost                          USD/hr      67.3
-    Design              Reactor volume                    m3       243
-                        Cycle time                        hr      12.6
+    Power               Rate                              kW      11.6
+                        Cost                          USD/hr     0.908
+    Chilled water       Duty                           kJ/hr -1.04e+07
+                        Flow                         kmol/hr  6.95e+03
+                        Cost                          USD/hr      51.8
+    Design              Reactor volume                    m3       246
+                        Batch time                        hr      12.6
                         Loading time                      hr      1.57
+                        Number of reactors                           8
+                        Reactor duty                   kJ/hr  1.04e+07
                         Cleaning and unloading time       hr         3
                         Working volume fraction                    0.9
-                        Number of reactors                           8
-    Purchase cost       Coolers                          USD   2.2e+05
-                        Reactors                         USD  1.86e+06
+    Purchase cost       Heat exchangers                  USD  4.12e+04
+                        Reactors                         USD  1.87e+06
                         Agitators                        USD  1.16e+05
-                        Cleaning in place                USD  7.05e+05
-    Total purchase cost                                  USD   2.9e+06
-    Utility cost                                      USD/hr      68.2
-    
+                        Cleaning in place                USD  8.87e+04
+    Total purchase cost                                  USD  2.12e+06
+    Utility cost                                      USD/hr      52.7
     
     References
     ----------
@@ -113,24 +132,8 @@ class Fermentation(Unit):
     .. [5] D. Humbird, R. Davis, L. Tao, C. Kinchin, D. Hsu, and A. Aden National. Renewable Energy Laboratory Golden, Colorado. P. Schoen, J. Lukas, B. Olthof, M. Worley, D. Sexton, and D. Dudgeon. Harris Group Inc. Seattle, Washington and Atlanta, Georgia. Process Design and Economics for Biochemical Conversion of Lignocellulosic Biomass to Ethanol Dilute-Acid Pretreatment and Enzymatic Hydrolysis of Corn Stover. May 2011. Technical Report NREL/TP-5100-47764
     
     """
-    _units = {'Reactor volume': 'm3',
-              'Cycle time': 'hr',
-              'Loading time': 'hr',
-              'Total dead time': 'hr'}
-    _N_ins = _N_outs = 2
-    _N_heat_utilities = 0
-    _has_power_utility = True
     line = 'Fermentation'    
     
-    #: [bool] If True, number of reactors (N) is chosen as to minimize installation cost in every simulation. Otherwise, N remains constant.
-    autoselect_N = False
-    
-    #: [float] Cleaning and unloading time (hr)
-    tau_0 = 3
-    
-    #: [float] Fraction of filled tank to total tank volume
-    V_wf = 0.9
-    
     #: tuple[float] Kinetic parameters for the kinetic model. Default constants are fitted for Oliveria's model (mu_m1, mu_m2, Ks1, Ks2, Pm1, Pm2, Xm, Y_PS, a)
     kinetic_constants = (0.31,  # mu_m1
                          1.01,  # mu_m2
@@ -142,27 +145,16 @@ class Fermentation(Unit):
                          0.45,  # Y_PS
                          0.18)  # a
     
-    def _get_design_specs(self):
-        return (('Cleaning and unloading time', self.tau_0, 'hr'),
-                ('Working volume fraction', self.V_wf, ''),
-                ('Number of reactors', self.N, ''))
-    
-    def __init__(self, ID='', ins=None, outs=(), *, 
-                 tau,  N, efficiency=0.9, iskinetic=False):
-        Unit.__init__(self, ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, 
+                 tau,  N=None, V=None, T=305.15, P=101325., Nmin=2, Nmax=36,
+                 efficiency=0.9, iskinetic=False):
+        BatchBioreactor.__init__(self, ID, ins, outs, thermo,
+                                 tau=tau, N=N, V=V, T=T, P=P, Nmin=Nmin, Nmax=Nmax)
         self.hydrolysis = Reaction('Sucrose + Water -> 2Glucose', 'Sucrose', 1.00)
         self.fermentation = Reaction('Glucose -> 2Ethanol + 2CO2',  'Glucose', efficiency)
         self.iskinetic = iskinetic
         self.efficiency = efficiency
-        self.tau = tau
-        self.N = N
-        self._cooler = hx = HXutility(None)
-        self.heat_utilities = hx.heat_utilities
-        hx._ins = hx._outs
-        vent, effluent = self.outs
-        hx._outs[0].T = effluent.T = vent.T = 305.15
-        vent.phase = 'g'
-
+        
     def _calc_efficiency(self, feed, tau):
         # Get initial concentrations
         y, e, s, w = feed.indices(['Yeast',
@@ -192,7 +184,8 @@ class Fermentation(Unit):
         
     @staticmethod
     def kinetic_model(z, t, *kinetic_constants):
-        """Return change of yeast, ethanol, and substrate concentration in kg/m3.
+        """
+        Return change of yeast, ethanol, and substrate concentration in kg/m3.
         
         Parameters
         ----------
@@ -231,16 +224,6 @@ class Fermentation(Unit):
         dPdt = (mu_P * X)
         dSdt =  - mu_S * X
         return (dXdt, dPdt, dSdt)
-       
-    @property
-    def N(self):
-        """[int] Number of reactors"""
-        return self._N
-    @N.setter
-    def N(self, N):
-        if N <= 1:
-            raise ValueError(f"number of reactors must be greater than 1, value {N} is infeasible")
-        self._N = N
 
     @property
     def efficiency(self):
@@ -249,13 +232,6 @@ class Fermentation(Unit):
     def efficiency(self, efficiency):
         self.fermentation.X = efficiency
 
-    @property
-    def tau(self):
-        return self._tau
-    @tau.setter
-    def tau(self, tau):
-        self._tau = tau
-
     def _run(self):
         vent, effluent = self.outs
         effluent.mix_from(self.ins)
@@ -264,43 +240,4 @@ class Fermentation(Unit):
         if self.iskinetic:
             self.fermentation.X = self._calc_efficiency(effluent, self._tau)
         self.fermentation(effluent_mol)
-        vent.copy_flow(effluent, ('CO2',), remove=True)
-        vent.recieve_vent(effluent)
-    
-    @property
-    def N_at_minimum_capital_cost(self):
-        cost_old = np.inf
-        self._N, N = 2, self._N
-        cost_new = self.purchase_cost
-        self._summary()
-        while cost_new < cost_old:
-            self._N += 1
-            self._summary()
-            cost_old = cost_new
-            cost_new = self.purchase_cost
-        self._N, N = N, self._N
-        return N - 1
-        
-    def _design(self):
-        v_0 = self.outs[1].F_vol
-        tau = self._tau
-        tau_0 = self.tau_0
-        Design = self.design_results
-        if self.autoselect_N:
-            self.autoselect_N = False
-            self._N = self.N_at_minimum_capital_cost
-            self.autoselect_N = True
-        N = self._N
-        Design.update(size_batch(v_0, tau, tau_0, N, self.V_wf))
-        hx = self._cooler
-        hx.outs[0].mol[:] = self.outs[0].mol/N 
-        hu = hx.heat_utilities[0]
-        hu(self.Hnet/N, self.outs[0].T)
-        hx._design(hu.duty)
-        hx._cost()
-        hu.duty *= N
-        hu.cost *= N
-        hu.flow *= N
-        self.purchase_costs['Coolers'] = self._cooler.purchase_costs['Heat exchanger'] * self._N
-        
-    
\ No newline at end of file
+        vent.receive_vent(effluent)
