@@ -1,24 +1,21 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 16:21:56 2018
-
-@author: yoelr
 """
 from .. import Unit, PowerUtility
 from thermosteam import MultiStream
-from math import pi, ceil
+from math import pi
 import numpy as np
-from .design_tools import (calculate_vacuum_system_power_and_cost,
-                           HNATable, ceil_half_step,
-                           compute_vessel_weight_and_wall_thickness,
-                           compute_Stokes_law_York_Demister_K_value,
-                           pressure_vessel_material_factors,
-                           material_densities_lb_per_ft3,)
+from . import design_tools as design
 from ._splitter import Splitter
 from ._hx import HX, HXutility
-from ..utils import bounds_warning
-import biosteam as bst
-from ._CAS import H2O_CAS
+from .._graphics import vertical_vessel_graphics
+from .design_tools import separations
 
 exp = np.exp
 ln = np.log
@@ -33,7 +30,7 @@ __all__ = ('Flash', 'SplitFlash', 'RatioFlash')
 
 # %% Flash
 
-class Flash(Unit):
+class Flash(design.PressureVessel, Unit):
     """
     Create an equlibrium based flash drum with the option of having light
     non-keys and heavy non-keys completly separate into their respective
@@ -48,17 +45,17 @@ class Flash(Unit):
     outs : stream sequence
         * [0] Vapor product
         * [1] Liquid product
-    P : float, optional
+    P=None : float
         Operating pressure [Pa].
-    Q : float, optional
+    Q=None : float
         Duty [kJ/hr].
-    T : float, optional
+    T=None : float
         Operating temperature [K].
-    V : float, optional
+    V=None : float
         Molar vapor fraction.
-    x : float, optional
+    x=None : float
         Molar composition of liquid (for binary mixtures).
-    y : float, optional
+    y=None : float
         Molar composition of vapor (for binary mixtures).
     vessel_material : str, optional
         Vessel construction material. Defaults to 'Carbon steel'.
@@ -69,8 +66,8 @@ class Flash(Unit):
         True if glycol groups are present in the mixture.
     has_amine_groups=False : bool
         True if amine groups are present in the mixture.
-    vessel_type='Default' : 'Horizontal', 'Vertical', or 'Default'
-        Vessel separation type. If 'Default', the vessel type will be chosen
+    vessel_type=None : 'Horizontal' or 'Vertical', optional
+        Vessel separation type. If not specified, the vessel type will be chosen
         according to heuristics.
     holdup_time=15.0 : float
         Time it takes to raise liquid to half full [min].
@@ -88,9 +85,8 @@ class Flash(Unit):
     Examples
     --------
     >>> from biosteam.units import Flash
-    >>> from thermosteam import Chemicals, Stream, settings
-    >>> chemicals = Chemicals(['Water', 'Glycerol'])
-    >>> settings.set_thermo(chemicals)
+    >>> from biosteam import Stream, settings
+    >>> settings.set_thermo(['Water', 'Glycerol'])
     >>> feed = Stream('feed', Glycerol=300, Water=1000)
     >>> bp = feed.bubble_point_at_P() # Feed at bubble point T
     >>> feed.T = bp.T
@@ -114,24 +110,23 @@ class Flash(Unit):
                         Glycerol  2.32
     [1] crude_glycerin
         phase: 'l', T: 137 degC, P: 1 atm
-        flow (kmol/hr): Water     42.4
+        flow (kmol/hr): Water     42.5
                         Glycerol  298
-    
     >>> F1.results()
-    Flash                                   Units            F1
-    Medium pressure steam Duty              kJ/hr      5.06e+07
-                          Flow            kmol/hr       1.4e+03
-                          Cost             USD/hr           385
-    Design                Vessel type                  Vertical
-                          Length               ft          23.5
-                          Diameter             ft           6.5
-                          Weight               lb      8.44e+03
-                          Wall thickness       in         0.375
-                          Material                 Carbon steel
-    Purchase cost         Flash               USD      5.82e+04
-                          Heat exchanger      USD      4.36e+04
-    Total purchase cost                       USD      1.02e+05
-    Utility cost                           USD/hr           385
+    Flash                                             Units            F1
+    Medium pressure steam Duty                        kJ/hr      5.05e+07
+                          Flow                      kmol/hr      1.39e+03
+                          Cost                       USD/hr           384
+    Design                Vessel type                            Vertical
+                          Length                         ft          16.5
+                          Diameter                       ft           8.5
+                          Weight                         lb         1e+04
+                          Wall thickness                 in         0.438
+                          Vessel material                    Carbon steel
+    Purchase cost         Vertical pressure vessel      USD      6.22e+04
+                          Heat exchanger                USD      4.35e+04
+    Total purchase cost                                 USD      1.06e+05
+    Utility cost                                     USD/hr           384
 
 
     References
@@ -150,45 +145,18 @@ class Flash(Unit):
         Cost Accounting and Capital Cost Estimation (Chapter 16)
     
     """
+    auxiliary_unit_names = ('heat_exchanger',)
     _units = {'Vertical vessel weight': 'lb',
               'Horizontal vessel weight': 'lb',
               'Length': 'ft',
               'Diameter': 'ft',
               'Weight': 'lb',
               'Wall thickness': 'in'}
+    _BM = {'Liquid-ring pump': 1.0,
+           **design.PressureVessel._BM}
+    _graphics = vertical_vessel_graphics 
     _N_outs = 2
     _N_heat_utilities = 0
-    
-    # Bare module factor
-    BM_horizontal = 3.05
-    BM_vertical = 4.16
-    @property
-    def BM(self):
-        vessel_type = self.SetType
-        if vessel_type == 'Vertical':
-            return self._BM_vertical
-        elif vessel_type == 'Horizontal':
-            return self._BM_horizontal
-        else:
-            raise AttributeError('vessel_type not defined')
-    
-    _bounds = {'Vertical vessel weight': (4200, 1e6),
-               'Horizontal vessel weight': (1e3, 9.2e5),
-               'Diameter': (3, 21),
-               'Vertical vessel length': (12, 40)}
-
-    @property
-    def vessel_material(self):
-        """Vessel construction material."""
-        return self._vessel_material
-    @vessel_material.setter
-    def vessel_material(self, material):
-        try: self._F_M = pressure_vessel_material_factors[material]
-        except KeyError:
-            raise ValueError(f"no material factor available for '{material}'; "
-                              "only the following materials are available: "
-                             f"{', '.join(pressure_vessel_material_factors)}")
-        self._vessel_material = material  
 
     def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
                  V=None, T=None, Q=None, P=None, y=None, x=None,
@@ -196,13 +164,15 @@ class Flash(Unit):
                  vacuum_system_preference='Liquid-ring pump',
                  has_glycol_groups=False,
                  has_amine_groups=False,
-                 vessel_type='Default',
+                 vessel_type=None,
                  holdup_time=15,
                  surge_time=7.5,
                  has_mist_eliminator=False):
         Unit.__init__(self, ID, ins, outs, thermo)
-        self._multistream = MultiStream(None)
-        self._heat_exchanger = None
+        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
+        self.heat_exchanger = hx = HXutility(None, None, ms, thermo=self.thermo) 
+        self.heat_utilities = hx.heat_utilities
+        hx._ins = self._ins
         
         #: Enforced molar vapor fraction
         self.V = V
@@ -252,7 +222,7 @@ class Flash(Unit):
         return self._P
     @P.setter
     def P(self, P):
-        if P < 101325 and not self.power_utility:
+        if P and P < 101325 and not self.power_utility:
             self.power_utility = PowerUtility()
         self._P = P
     
@@ -262,77 +232,36 @@ class Flash(Unit):
         return self._Q
     @Q.setter
     def Q(self, Q):
-        if Q == 0:
-            self._heat_exchanger = None
-        elif not self._heat_exchanger:
-            self._heat_exchanger = he = HXutility(None, outs=None) 
-            self.heat_utilities = he.heat_utilities
-            he._ins = self._ins
-            he._outs[0] = self._multistream
         self._Q = Q
 
-    def _run(self):
-        # Unpack
+    def _default_vessel_type(self):
         vap, liq = self.outs
-        feed = self.ins[0]
+        F_mass_vap = vap.F_mass
+        F_mass_liq = liq.F_mass 
+        assert F_mass_liq, "no liquid effluent; flash vessel must have liquid to default vessel type"
+        return 'Vertical'if F_mass_vap / F_mass_liq > 0.1 else 'Horizontal'
 
-        # Vapor Liquid Equilibrium
-        ms = self._multistream
-        ms.empty()
-        ms.imol['l'] = feed.mol
-        ms.T = feed.T
-        Q = self.Q
-        H = feed.H + Q if Q is not None else None
-        ms.vle(self.P, H, self.T, self.V, self.x, self.y)
-
-        # Set Values
-        vap.phase = 'g'
-        liq.phase = 'l'
-        vap.mol[:] = ms.imol['g']
-        liq.mol[:] = ms.imol['l']
-        vap.T = liq.T = ms.T
-        vap.P = liq.P = ms.P
+    def _run(self):
+        separations.vle(self.ins[0], *self.outs, self.T, self.P, self.V, 
+                        self.Q, self.x, self.y, self._multi_stream)
 
     def _design(self):
-        # Set horizontal or vertical vessel
         vessel_type = self.vessel_type
-        if vessel_type == 'Default':
-            vap, liq = self.outs
-            isVertical = vap.F_mass/liq.F_mass > 0.2
-        elif vessel_type == 'Vertical':
-            isVertical = True
-        elif vessel_type == 'Horizontal':
-            isVertical = False
-        else:
-            raise ValueError( f"vessel_type must be either 'Default', 'Horizontal', 'Vertical', not '{self.vessel_type}'")
-        self._isVertical = isVertical
-
-        # Run vertical or horizontal design
-        if isVertical: self._vertical()
-        else: self._horizontal()
-        if self._heat_exchanger: self._heat_exchanger._design()
-        self.design_results['Material'] = self._vessel_material
+        if vessel_type == 'Vertical': 
+            args = self._vertical_vessel_pressure_diameter_and_length()
+        elif vessel_type == 'Horizontal': 
+            args = self._horizontal_vessel_pressure_diameter_and_length()
+        else: raise RuntimeError('unknown vessel type')
+        self.heat_exchanger._summary()
+        self.design_results.update(
+            self._vessel_design(*args)
+        )
 
     def _cost(self):
-        Design = self.design_results
-        W = Design['Weight']
-        D = Design['Diameter']
-        L = Design['Length']
-        
-        # C_v: Vessel cost
-        # C_pl: Platforms and ladders cost
-        if self._isVertical:
-            C_v = exp(7.1390 + 0.18255*ln(W) + 0.02297*ln(W)**2)
-            C_pl = 410*D**0.7396*L**0.70684
-        else:
-            C_v = exp(5.6336 - 0.4599*ln(W) + 0.00582*ln(W)**2)
-            C_pl = 2275*D**0.20294
-            
-        self.purchase_costs['Flash'] = bst.CE/567*(self._F_M * C_v + C_pl)
-        if self._heat_exchanger:
-            hx = self._heat_exchanger
-            hx._cost()
-            self.purchase_costs.update(hx.purchase_costs)
+        D = self.design_results
+        self.purchase_costs.update(
+            self._vessel_purchase_cost(D['Weight'], D['Diameter'], D['Length'])
+        )
         self._cost_vacuum()
 
     def _cost_vacuum(self):
@@ -363,7 +292,7 @@ class Flash(Unit):
             F_mass = vapor.F_mass
             F_vol = vapor.F_vol
         
-        power, cost = calculate_vacuum_system_power_and_cost(
+        power, cost = design.compute_vacuum_system_power_and_cost(
                           F_mass, F_vol, P, volume, self.vacuum_system_preference)
         self.purchase_costs['Liquid-ring pump'] = cost
         self.power_utility(power)
@@ -373,7 +302,7 @@ class Flash(Unit):
         vap, liq = self._outs
         rhov = vap.get_property('rho', 'lb/ft3')
         rhol = liq.get_property('rho', 'lb/ft3')
-        P = liq.P*0.000145  # Pressure (psi)
+        P = liq.get_property('P', 'psi')  # Pressure (psi)
 
         vessel_type = self.vessel_type
         Th = self.holdup_time
@@ -385,7 +314,7 @@ class Flash(Unit):
         Qll = liq.get_total_flow('ft^3 / min')
 
         # Calculate Ut and set Uv
-        K = compute_Stokes_law_York_Demister_K_value(P)
+        K = design.compute_Stokes_law_York_Demister_K_value(P)
 
         # Adjust K value
         if not has_mist_eliminator and vessel_type == 'Vertical': K /= 2
@@ -402,35 +331,35 @@ class Flash(Unit):
         Vs = Ts*Qll
         return rhov, rhol, P, Th, Ts, has_mist_eliminator, Qv, Qll, Ut, Uv, Vh, Vs
 
-    def _vertical(self):
+    def _vertical_vessel_pressure_diameter_and_length(self):
         rhov, rhol, P, Th, Ts, has_mist_eliminator, Qv, Qll, Ut, Uv, Vh, Vs = self._design_parameters()
 
         # Calculate internal diameter, Dvd
         Dvd = (4.0*Qv/(pi*Uv))**0.5
         if has_mist_eliminator:
-            D = ceil_half_step(Dvd + 0.4)
+            D = design.ceil_half_step(Dvd + 0.4)
         else:
-            D = ceil_half_step(Dvd)
+            D = design.ceil_half_step(Dvd)
 
         # Obtaining low liquid level height, Hlll
         Hlll = 0.5
         if P < 300:
             Hlll = 1.25
 
-        # Calculate the height from Hlll to  Normal liq level, Hnll
+        # Calculate the height from Hlll to Normal liq level, Hnll
         Hh = Vh/(pi/4.0*Dvd**2)
-        Hh = 1.0 if Hh < 1.0 else ceil_half_step(Hh)
+        if Hh < 1.0: Hh = 1.0
 
         # Calculate the height from Hnll to  High liq level, Hhll
         Hs = Vs/(pi/4.0*Dvd**2)
-        Hs = 0.5 if Hs < 0.5 else ceil_half_step(Hs)
+        if Hs < 0.5: Hs = 0.5
 
         # Calculate dN
         Qm = Qll + Qv
         lamda = Qll/Qm
         rhoM = rhol*lamda + rhov*(1-lamda)
         dN = (4*Qm/(pi*60.0/(rhoM**0.5)))**0.5
-        dN = ceil_half_step(dN)
+        dN = design.ceil_half_step(dN)
 
         # Calculate Hlin, assume with inlet diverter
         Hlin = 1.0 + dN
@@ -439,38 +368,20 @@ class Flash(Unit):
         Hv = 0.5*Dvd
         Hv2 = (2.0 if has_mist_eliminator else 3.0) + dN/2.0
         if Hv2 < Hv: Hv = Hv2
-        Hv = ceil(Hv)
+        Hv = Hv
 
         # Calculate total height, Ht
         Hme = 1.5 if has_mist_eliminator else 0.0
         Ht = Hlll + Hh + Hs + Hlin + Hv + Hme
-        Ht = ceil_half_step(Ht)
-
-        # Check if LD is between 1.5 and 6.0
-        while True:
-            LD = Ht/D
-            if (LD < 1.5): D -= 0.5
-            elif (LD > 6.0): D += 0.5
-            else: break
-
-        # Calculate Vessel weight and wall thickness
-        rho_M = material_densities_lb_per_ft3[self._vessel_material]
-        VW, VWT = compute_vessel_weight_and_wall_thickness(P, D, Ht, rho_M)
+        Ht = design.ceil_half_step(Ht)
 
         # Find maximum and normal liquid level
         # Hhll = Hs + Hh + Hlll
         # Hnll = Hh + Hlll
 
-        Design = self.design_results
-        bounds_warning(self, 'Vertical vessel weight', VW, 'lb', self._bounds['Vertical vessel weight'])
-        bounds_warning(self, 'Vertical vessel length', Ht, 'ft', self._bounds['Vertical vessel length'])
-        Design['Vessel type'] = 'Vertical'
-        Design['Length'] = Ht     # ft
-        Design['Diameter'] = D    # ft
-        Design['Weight'] = VW     # lb
-        Design['Wall thickness'] = VWT  # in
+        return P, D, Ht
         
-    def _horizontal(self):
+    def _horizontal_vessel_pressure_diameter_and_length(self):
         rhov, rhol, P, Th, Ts, has_mist_eliminator, Qv, Qll, Ut, Uv, Vh, Vs = self._design_parameters()
 
         # Initialize LD
@@ -485,7 +396,7 @@ class Flash(Unit):
         if D <= 4.0:
             D = 4.0
         else:
-            D = ceil_half_step(D)
+            D = design.ceil_half_step(D)
 
         for outerIter in range(50):
             At = pi*(D**2)/4.0 # Total area
@@ -494,15 +405,15 @@ class Flash(Unit):
             Hlll = round(0.5*D + 7.0)  
             Hlll = Hlll/12.0 # D is in ft but Hlll is in inches
             X = Hlll/D
-            Y = HNATable(1, X)
+            Y = design.HNATable(1, X)
             Alll = Y*At
 
             # Calculate the Vapor disengagement area, Av
             Hv = 0.2*D
             if has_mist_eliminator and Hv <= 2.0: Hv = 2.0
             elif Hv <= 1.0: Hv = 1.0
-            else: Hv = ceil_half_step(Hv)
-            Av = HNATable(1, Hv/D)*At
+            else: Hv = design.ceil_half_step(Hv)
+            Av = design.HNATable(1, Hv/D)*At
             
             # Calculate minimum length for surge and holdup
             L = (Vh + Vs)/(At - Av - Alll)
@@ -521,8 +432,8 @@ class Flash(Unit):
                     elif not has_mist_eliminator and Hv <= 1.0: Hv = 1.0
                     else: Hv -= 0.5
                 else: break
-                Av = HNATable(1, Hv/D)*At
-                Alll = HNATable(1, Hlll/D)*At
+                Av = design.HNATable(1, Hv/D)*At
+                Alll = design.HNATable(1, Hlll/D)*At
                 Li = (Vh + Vs)/(At - Av - Alll)
                 Phi = Hv/Uv
                 Uva = Qv/Av
@@ -547,10 +458,6 @@ class Flash(Unit):
             elif (LD > 6.0): D += 0.5
             else: break
 
-        # Calculate vessel weight and wall thickness
-        rho_M = material_densities_lb_per_ft3[self._vessel_material]
-        VW, VWT = compute_vessel_weight_and_wall_thickness(P, D, L, rho_M)
-
         # # To check minimum Hv value
         # if int(has_mist_eliminator) == 1 and Hv <= 2.0:
         #     Hv = 2.0
@@ -566,36 +473,30 @@ class Flash(Unit):
         # Y = HNATable(2, X)
         # Hnll = Y*D
         
-        Design = self.design_results
-        bounds_warning(self, 'Horizontal vessel weight', VW, 'lb', self._bounds['Horizontal vessel weight'])
-        Design['Vessel type'] = 'Horizontal'
-        Design['Length'] = L  # ft
-        Design['Diameter'] = D  # ft
-        Design['Weight'] = VW  # lb
-        Design['Wall thickness'] = VWT  # in    
-        
-    def _end_decorated_cost_(self):
-        if self.heat_utilities: self.heat_utilities[0](self.Hnet, self.T)
-    
+        return P, D, L
+
 
 # %% Special
 
 class SplitFlash(Flash):
     line = 'Flash' 
     
-    def __init__(self, ID='', ins=None, outs=(), *, split,
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, split,
                  order=None, T=None, P=None, Q=None,
                  vessel_material='Carbon steel',
                  vacuum_system_preference='Liquid-ring pump',
                  has_glycol_groups=False,
                  has_amine_groups=False,
-                 vessel_type='Default',
+                 vessel_type=None,
                  holdup_time=15,
                  surge_time=7.5,
                  has_mist_eliminator=False):
-        Splitter.__init__(self, ID, ins, outs, split=split, order=order)
-        self._multistream = MultiStream(None)
-        self._heat_exchanger = None
+        Splitter.__init__(self, ID, ins, outs, thermo, split=split, order=order)
+        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
+        
+        #: [HXutility] Heat exchanger if needed.
+        self.heat_exchanger = hx = HXutility(None, None, ms, thermo=self.thermo)
+        hx._ins = self._ins
         self.T = T #: Operating temperature (K)
         self.P = P #: Operating pressure (Pa)
         self.Q = Q #: Duty (kJ/hr)
@@ -639,9 +540,8 @@ class SplitFlash(Flash):
         bot.P = top.P = self.P
 
     def _design(self):
-        if self._heat_exchanger:
-            self._heat_exchanger.outs[0] = ms = self._multistream
-            ms.mix_from(self.outs)
+        self.heat_exchanger.outs[0] = ms = self._multi_stream
+        ms.mix_from(self.outs)
         super()._design()
     
 
@@ -652,6 +552,9 @@ class RatioFlash(Flash):
                  K_chemicals, Ks, top_solvents=(), top_split=(),
                  bot_solvents=(), bot_split=()):
         Unit.__init__(self, ID, ins, outs)
+        self._multi_stream = ms = MultiStream(None, thermo=self.thermo)
+        #: [HXutility] Heat exchanger if needed.
+        self.heat_exchanger = HXutility(None, None, ms, thermo=self.thermo)
         self.K_chemicals = K_chemicals
         self.Ks = Ks
         self.top_solvents = top_solvents
@@ -679,6 +582,10 @@ class RatioFlash(Flash):
         top.T, top.P = feed.T, feed.P
         bot.T, bot.P = feed.T, feed.P
 
+    def _design(self):
+        self.heat_exchanger.outs[0] = ms = self._multi_stream
+        ms.mix_from(self.outs)
+        super()._design()
 
 # %% Single Component
 
@@ -690,7 +597,7 @@ class Evaporator_PQ(Unit):
         return self._P
     @P.setter
     def P(self, P):
-        water = getattr(self.chemicals, H2O_CAS)
+        water = getattr(self.chemicals, '7732-18-5')
         self._T = T = water.Tsat(P)
         self._Hvap = water.Hvap(T)
         self._P = P
@@ -699,7 +606,7 @@ class Evaporator_PQ(Unit):
         return self._T
     @T.setter
     def T(self, T):
-        water = getattr(self.chemicals, H2O_CAS)
+        water = getattr(self.chemicals, '7732-18-5')
         self._P = water.Psat(T)
         self._Hvap = water.Hvap(T)
         self._T = T
@@ -707,8 +614,8 @@ class Evaporator_PQ(Unit):
     def V(self):
         return self._V
     
-    def __init__(self, ID='', ins=None, outs=(), *, Q=0, P=101325):
-        super().__init__(ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, Q=0, P=101325):
+        super().__init__(ID, ins, outs, thermo)
         self.Q = Q
         self.P = P
         self._V = None
@@ -719,7 +626,7 @@ class Evaporator_PQ(Unit):
         
         # Optional if Q also comes from condensing a side stream
         Q = self.Q
-        if utility_liquid:
+        if not utility_vapor.isempty():
             utility_liquid.copy_like(utility_vapor)
             utility_liquid.phase = 'l'
             Q += utility_vapor.Hvap
@@ -730,20 +637,23 @@ class Evaporator_PQ(Unit):
         vapor.P = liquid.P = self.P
         liquid.phase = 'l'
         vapor.phase = 'g'
-        liquid.copy_flow(feed, IDs=H2O_CAS, exclude=True)
+        liquid.copy_flow(feed)
         
         # Energy balance to find vapor fraction
-        f = feed.imol[H2O_CAS]
+        f = feed.imol['7732-18-5']
         H = feed_H + Q - liquid.H
-        V = H/(f * self._Hvap)
-        if V < 0:
+        if f:
+            V = H/(f * self._Hvap)
+            if V < 0:
+                V = 0
+            elif V > 1:
+                V = 1
+        else:
             V = 0
-        elif V > 1:
-            V = 1
         evaporated = f * V
-        vapor.imol[H2O_CAS] = evaporated
-        liquid.imol[H2O_CAS] = (1-V)*f
-        self._Q = Q
+        vapor.imol['7732-18-5'] = evaporated
+        liquid.imol['7732-18-5'] = (1-V)*f
+        self.design_results['Heat transfer'] = Q
         self._V = V
 
 
@@ -755,7 +665,7 @@ class Evaporator_PV(Unit):
         return self._P
     @P.setter
     def P(self, P):
-        water = getattr(self.chemicals, H2O_CAS)
+        water = getattr(self.chemicals, '7732-18-5')
         self._T = water.Tsat(P)
         self._P = P
     @property
@@ -763,12 +673,12 @@ class Evaporator_PV(Unit):
         return self._T
     @T.setter
     def T(self, T):
-        water = getattr(self.chemicals, H2O_CAS)
+        water = getattr(self.chemicals, '7732-18-5')
         self._P = water.Psat(T)
         self._T = T
     
-    def __init__(self, ID='', ins=None, outs=(), *, V=0.5, P=101325):
-        super().__init__(ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, V=0.5, P=101325):
+        super().__init__(ID, ins, outs, thermo)
         self.V = V
         self.P = P
 
@@ -776,7 +686,8 @@ class Evaporator_PV(Unit):
         feed = self.ins[0]
         vapor, liquid = self.outs
         vapor.T = liquid.T = self.T
-        H2O_index = self.chemicals.index(H2O_CAS)
+        vapor.P = liquid.P = self.P
+        H2O_index = self.chemicals.index('7732-18-5')
         water_mol = feed.mol[H2O_index]
         vapor.mol[H2O_index] = self.V * water_mol
         liquid_mol = liquid.mol
