@@ -1,10 +1,14 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 14:26:41 2018
-
-@author: yoelr
 """
 from .. import Unit
+from .._graphics import mixer_graphics
 
 __all__ = ('Mixer',)
 
@@ -14,20 +18,18 @@ class Mixer(Unit):
     
     Parameters
     ----------
-    ins :
-        Inlet streams to be mixed.
-    outs :
-        Mixed outlet stream.
+    ins : streams
+        Inlet fluids to be mixed.
+    outs : stream
+        Mixed outlet fluid.
     Examples
     --------
     Mix two streams:
     
-    >>> from biosteam import units
-    >>> import thermosteam as tmo
-    >>> chemicals = tmo.Chemicals(['Ethanol', 'Water'])
-    >>> tmo.settings.set_thermo(chemicals)
-    >>> s1 = tmo.Stream('s1', Water=20, T=350)
-    >>> s2 = tmo.Stream('s2', Ethanol=30, T=300)
+    >>> from biosteam import units, settings, Stream
+    >>> settings.set_thermo(['Ethanol', 'Water'])
+    >>> s1 = Stream('s1', Water=20, T=350)
+    >>> s2 = Stream('s2', Ethanol=30, T=300)
     >>> M1 = units.Mixer('M1', ins=(s1, s2), outs='s3')
     >>> M1.simulate()
     >>> M1.show()
@@ -47,9 +49,9 @@ class Mixer(Unit):
     
     
     """
-    _has_cost = False
+    _graphics = mixer_graphics
     _N_outs = 1
-    _N_ins = 5
+    _N_ins = 2
     _ins_size_is_fixed = False
     
     def _run(self):
