@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:17:05 2018
-
-@author: yoelr
 """
 from ._splitter import Splitter
 from .decorators import cost
@@ -34,10 +37,9 @@ class MolecularSieve(Splitter):
     
     Examples
     --------
-    >>> from thermosteam import Chemicals, Stream, settings
+    >>> from biosteam import Stream, settings
     >>> from biosteam.units import MolecularSieve
-    >>> chemicals = Chemicals(['Water', 'Ethanol'])
-    >>> settings.set_thermo(chemicals)
+    >>> settings.set_thermo(['Water', 'Ethanol'])
     >>> feed = Stream('feed', flow=(75.7, 286), phase='g')
     >>> bp = feed.bubble_point_at_T()
     >>> feed.T = bp.T
@@ -71,7 +73,7 @@ class MolecularSieve(Splitter):
     Power               Rate            kW      14.2
                         Cost        USD/hr      1.11
     Low pressure steam  Duty         kJ/hr  3.21e+06
-                        Flow       kmol/hr      82.6
+                        Flow       kmol/hr      82.7
                         Cost        USD/hr      19.7
     Chilled water       Duty         kJ/hr -1.18e+05
                         Flow       kmol/hr       123
@@ -81,7 +83,6 @@ class MolecularSieve(Splitter):
     Total purchase cost                USD  6.85e+05
     Utility cost                    USD/hr      21.4
     
-    
     References
     ----------
     .. [1] Process Design and Economics for Biochemical Conversion of
@@ -97,10 +98,6 @@ class MolecularSieve(Splitter):
     _units = {'Flow rate': 'kg/hr'}
     def __init__(self, ID='', ins=None, outs=(), *, order=None, split):
         Splitter.__init__(self, ID, ins, outs, order=order, split=split)
-        
-    def _setup(self):
-        s1_out, s2_out = self.outs
-        s1_out.phase = s2_out.phase = 'g'
 
     def _design(self):
         self.design_results['Flow rate'] = flow = self._outs[1].F_mass
