@@ -1,19 +1,29 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 21:43:13 2018
-
-@author: yoelr
 """
 import numpy as np
 import biosteam as bst
 from .. import Unit
-from . import Mixer, HXutility
+from ._mixer import Mixer
+from ._hx import HXutility
 from ._flash import Evaporator_PV, Evaporator_PQ
-from .design_tools import calculate_vacuum_system_power_and_cost
-from thermosteam import Stream, settings
-from flexsolve import IQ_interpolation
+from .design_tools import (
+    compute_vacuum_system_power_and_cost,
+    compute_heat_transfer_area
+)    
+from thermosteam import MultiStream, Stream, settings
+import flexsolve as flx
 from warnings import warn
-import ht
+from .design_tools import heat_transfer as ht
+
+__all__ = ('MultiEffectEvaporator',)
+
 log = np.log
 exp = np.exp
 
@@ -42,12 +52,20 @@ evaporators = {'Horizontal tube':
 
 
 class MultiEffectEvaporator(Unit):
-    """Creates evaporatorators with pressures given by P (a list of pressures). Adjusts first evaporator vapor fraction to satisfy an overall fraction evaporated. All evaporators after the first have zero duty. Condenses the vapor coming out of the last evaporator. Pumps all liquid streams to prevent back flow in later parts. All liquid evaporated is ultimately recondensed. Cost is based on required heat transfer area. Vacuum system is based on air leakage. Air leakage is based on volume, as given by residence time `tau` and flow rate to each evaporator.
+    """
+    Creates evaporatorators with pressures given by P (a list of pressures). 
+    Adjusts first evaporator vapor fraction to satisfy an overall fraction
+    evaporated. All evaporators after the first have zero duty. Condenses
+    the vapor coming out of the last evaporator. Pumps all liquid streams
+    to prevent back flow in later parts. All liquid evaporated is ultimately
+    recondensed. Cost is based on required heat transfer area. Vacuum system
+    is based on air leakage. Air leakage is based on volume, as given by
+    residence time `tau` and flow rate to each evaporator.
 
     Parameters
     ----------
     ins : stream
-        Inlet stream.
+        Inlet.
     outs : stream sequence
         * [0] Solid-rich stream.
         * [1] Condensate stream.
@@ -61,12 +79,14 @@ class MultiEffectEvaporator(Unit):
             Liquid pressure after pumping (Pa).
     
     """
+    line = 'Multi-Effect Evaporator'
     _units = {'Area': 'm^2',
               'Volume': 'm^3'}
+    _BM = {'Evaporators': 2.45,
+           'Liquid-ring pump': 1.0,
+           'Condenser': 3.17}
     _N_outs = 2
     _N_heat_utilities = 2
-    BM = 2.45
-    line = 'Multi-Effect Evaporator'
 
     #: Residence time (hr)
     tau = 0.30
@@ -90,8 +110,8 @@ class MultiEffectEvaporator(Unit):
             raise ValueError(f"Type must be one of the following: {dummy}")
         self._Type = evap_type
 
-    def __init__(self, ID='', ins=None, outs=(), *, P, V):
-        Unit.__init__(self, ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, P, V):
+        Unit.__init__(self, ID, ins, outs, thermo)
         # Unpack
         out_wt_solids, liq = self.outs
         self.V = V #: [float] Overall molar fraction of component evaporated.
@@ -116,7 +136,6 @@ class MultiEffectEvaporator(Unit):
                            'condenser': condenser,
                            'mixer': mixer}
         
-
     def _run(self):
         out_wt_solids, liq = self.outs
         ins = self.ins
@@ -144,15 +163,17 @@ class MultiEffectEvaporator(Unit):
             for evap in other_evaporators:
                 evap._run()
                 v_overall += (1-v_overall) * evap.V
-            return v_overall
+            return v_overall - self.V
         
         x0 = 0.0001
         x1 = 0.9990
         y0 = compute_overall_vapor_fraction(x0)
         y1 = compute_overall_vapor_fraction(x1)
-        self._V1 = IQ_interpolation(compute_overall_vapor_fraction,
-                                    x0, x1, y0, y1, self._V1, self.V, 
-                                    xtol=0.0001, ytol=0.001)
+        self._V1 = flx.IQ_interpolation(compute_overall_vapor_fraction,
+                                        x0, x1, y0, y1, self._V1, 
+                                        xtol=0.0001, ytol=0.001,
+                                        checkroot=False)
+        
         # Condensing vapor from last effector
         outs_vap = evaporators[-1].outs[0]
         condenser.ins[:] = [outs_vap]
@@ -170,6 +191,12 @@ class MultiEffectEvaporator(Unit):
         mixer._run()
         liq.copy_like(mixer.outs[0])
         
+        mixed_stream = MultiStream(thermo=self.thermo)
+        mixed_stream.copy_flow(self.ins[0])
+        mixed_stream.vle(P=evaporators[-1].P, V=self.V)
+        out_wt_solids.mol = mixed_stream.imol['l']
+        liq.mol = mixed_stream.imol['g']
+        
     def _design(self):
         # This functions also finds the cost
         A_range, C_func, U, _ = self._evap_data
@@ -187,26 +214,26 @@ class MultiEffectEvaporator(Unit):
         Tco = first_evaporator.outs[0].T
         hu(duty, Tci, Tco)
         Th = hu.inlet_utility_stream.T
-        LMTD = ht.LMTD(Th, Th, Tci, Tco)
+        LMTD = ht.compute_LMTD(Th, Th, Tci, Tco)
         ft = 1
-        A = abs(HXutility._calc_area(LMTD, U, Q, ft))
+        A = abs(compute_heat_transfer_area(LMTD, U, Q, ft))
         self._evap_costs = evap_costs = [C_func(A, CE)]
         
         # Find condenser requirements
         condenser = components['condenser']
         condenser._design()
         condenser._cost()
-        Cost['Condenser'] = condenser.purchase_costs['Heat exchanger']
+        Cost['Condenser'] = condenser.purchase_cost
         
         # Find area and cost of evaporators
         As = [A]
         A_min, A_max = A_range
         for evap in evaporators[1:]:
-            Q = evap._Q
+            Q = evap.design_results['Heat transfer']
             Tc = evap.outs[0].T
             Th = evap.outs[2].T
             LMTD = Th - Tc
-            A = HXutility._calc_area(LMTD, U, Q, ft)
+            A = compute_heat_transfer_area(LMTD, U, Q, ft)
             As.append(A)
             if settings.debug and not A_min < A < A_max:
                 warn(f'area requirement ({A}) is out of range, {A_range}')
@@ -217,11 +244,11 @@ class MultiEffectEvaporator(Unit):
         Cost['Evaporators'] = sum(evap_costs)
         
         # Calculate power
-        power, cost = calculate_vacuum_system_power_and_cost(
+        power, cost = compute_vacuum_system_power_and_cost(
             F_mass=0, F_vol=0, P_suction=evap.outs[0].P,
             vessel_volume=total_volume,
             vacuum_system_preference='Liquid-ring pump')
-        Cost['Vacuum liquid-ring pump'] = cost
+        Cost['Liquid-ring pump'] = cost
         self.power_utility(power)
         
         
