@@ -1,18 +1,22 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:15:20 2018
-
-@author: yoelr
 """
 from ._solids_separator import SolidsSeparator
-from .design_tools import calculate_vacuum_system_power_and_cost
+from .design_tools import compute_vacuum_system_power_and_cost
 import numpy as np
 import biosteam as bst
 
 __all__ = ('RotaryVacuumFilter', 'RVF')
 
 class RotaryVacuumFilter(SolidsSeparator):
-    """Create a RotaryVacuumFilter object.
+    """
+    Create a RotaryVacuumFilter object.
     
     Parameters
     ----------
@@ -26,8 +30,8 @@ class RotaryVacuumFilter(SolidsSeparator):
                        Fraction of water in retentate.
     
     """
-    _has_power_utility = True
-    BM = 2.32
+    _BM = {'Vessels': 2.32,
+           'Liquid-ring pump': 1.0}
     
     #: Revolutions per second
     rps = 20/3600
@@ -63,7 +67,7 @@ class RotaryVacuumFilter(SolidsSeparator):
         Design['Individual area'] = iArea
         logArea = np.log(iArea)
         Cost = np.exp(11.796-0.1905*logArea+0.0554*logArea**2)
-        self.purchase_costs['Cost of vessels'] = N_vessels*Cost*bst.CE/567
+        self.purchase_costs['Vessels'] = N_vessels*Cost*bst.CE/567
     
     def _power(self, area, N_vessels):
         s_cake, s_vacuumed = self.outs
@@ -92,7 +96,7 @@ class RotaryVacuumFilter(SolidsSeparator):
         # Assume same volume of air comes in as volume of liquid
         F_vol = s_vacuumed.F_vol
         F_mass = F_vol * 1.2041 # multiply by density of air kg/m3 
-        work_vacuum, self.purchase_costs['Liquid-ring pump'] = calculate_vacuum_system_power_and_cost(
+        work_vacuum, self.purchase_costs['Liquid-ring pump'] = compute_vacuum_system_power_and_cost(
                 F_mass, F_vol, self.P_suction, vessel_volume)
         #power = work_rot/self.power_efficiency/1000 + work_vacuum # kW
         self.power_utility(work_vacuum)
