@@ -1,17 +1,25 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Aug 23 22:49:58 2018
-
-@author: yoelr
 """
 from .. import Unit
 from .decorators import cost
 from thermosteam.reaction import Reaction, ParallelReaction
 
+__all__ = ('Transesterification',)
+
 @cost('Volume', 'Reactor',
       CE=525.4, cost=15000, n=0.55, kW=1.5, BM=4.3,)
 class Transesterification(Unit):
-    """Create a transesterification reactor that converts 'Lipid' and 'Methanol' to 'Biodiesel' and 'Glycerol'. Finds the amount of catalyst 'NaOCH3' required and consumes it to 'NaOH' and 'Methanol'.
+    """
+    Create a transesterification reactor that converts 'Lipid' and 'Methanol'
+    to 'Biodiesel' and 'Glycerol'. Finds the amount of catalyst 'NaOCH3'
+    required and consumes it to produce 'NaOH' and 'Methanol'.
     
     Parameters
     ----------
@@ -21,13 +29,13 @@ class Transesterification(Unit):
     outs : stream
         Reactor effluent.
     efficiency : float
-        Efficiency of conversion (on a 'Lipid' basis)
+        Efficiency of conversion (on a 'Lipid' basis).
     methanol2lipid : float
-        Methanol feed to lipid molar ratio
+        Methanol feed to lipid molar ratio.
     catalyst_molfrac : float
-        Catalyst to methanol molar ratio
+        Catalyst to methanol molar ratio.
     T : float
-        Operating temperature (K)
+        Operating temperature [K].
     
     """
     _bounds = {'Volume': (0.1, 20)}
@@ -45,7 +53,7 @@ class Transesterification(Unit):
     def __init__(self, ID='', ins=None, outs=(), *,
                  efficiency, methanol2lipid, T, catalyst_molfrac):
         Unit.__init__(self, ID, ins, outs)
-        #: [ParallelReaction] Transesterification and catalyst consumption reaction
+        #: [:class:`~thermosteam.ParallelReaction`] Transesterification and catalyst consumption reaction
         self.reaction = ParallelReaction([
           Reaction('Lipid + 3Methanol -> 3Biodiesel + Glycerol',
                    reactant='Lipid',  X=efficiency),
@@ -58,7 +66,7 @@ class Transesterification(Unit):
         self._lipid_index, self._methanol_index, self._catalyst_index = \
                 chemicals.get_index(('Lipid', 'Methanol', 'NaOCH3'))
         self._methanol2lipid = methanol2lipid
-        self.T = T #: Operation temperature (K).
+        self.T = T #: Operating temperature (K).
     
     @property
     def tau(self):
