@@ -1,27 +1,42 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Jan  2 01:14:00 2020
+General functional algorithms for the design of pumps and motors.
 
-@author: yoelr
 """
-from math import log as ln
-from fluids.pump import nema_sizes_hp
+import numpy as np
+from flexsolve import njitable
 
 __all__ = ('brake_efficiency', 'motor_efficiency', 'pump_efficiency',
            'nearest_NEMA_motor_size')
 
+#: [tuple] All NEMA motor sizes in increasing order, in horsepower.
+nema_sizes_hp = (0.25, 0.3333333333333333, 0.5, 0.75, 1.0, 1.5, 2.0,
+                 3.0, 4.0, 5.0, 5.5, 7.5, 10.0, 15.0, 20.0, 25.0, 30.0,
+                 40.0, 50.0, 60.0, 75.0, 100.0, 125.0, 150.0, 175.0, 200.0, 
+                 250.0, 300.0, 350.0, 400.0, 450.0, 500.0)
+
+
+@njitable(cache=True)
 def brake_efficiency(q):
     """Return brake efficiency given flow rate in gpm."""
     if q < 50: q = 50
     elif q > 5000: q = 5000
-    return -0.316 + 0.24015*ln(q) - 0.01199*ln(q)**2
+    return -0.316 + 0.24015*np.log(q) - 0.01199*np.log(q)**2
 
+@njitable(cache=True)
 def motor_efficiency(Pb):
     """Return motor efficiency given brake power in hp."""
     if Pb < 1: Pb = 1
     elif Pb > 1500: Pb = 1500
-    return 0.8 + 0.0319*ln(Pb) - 0.00182*ln(Pb)**2
+    return 0.8 + 0.0319*np.log(Pb) - 0.00182*np.log(Pb)**2
 
+@njitable(cache=True)
 def pump_efficiency(q, p):
     """Return pump efficiency.
     
@@ -40,7 +55,8 @@ def nearest_NEMA_motor_size(power):
     for nearest_power in nema_sizes_hp:
         if nearest_power >= power: return nearest_power
     raise ValueError(f'no NEMA motor size bigger than {power} hp')
-    
+
+@njitable(cache=True)    
 def calculate_NPSH(P_suction, P_vapor, rho_liq):
     """Return NPSH in ft given suction and vapor pressure in Pa and density in kg/m^3."""
     # Note: NPSH = (P_suction - P_vapor)/(rho_liq*gravity)
